@@ -25,6 +25,9 @@ from Products.ATContentTypes.configuration import zconf
 from Products.validation.config import validation
 from Products.validation.validators.SupplValidators import MaxSizeValidator
 from Products.validation import V_REQUIRED
+from zope.component import queryUtility
+from redturtle.smartlink.interfaces.utility import ISmartlinkConfig
+from Products.CMFPlone.utils import getToolByName
 
 LinkSchema = ATLinkSchema.copy() + atapi.Schema((
 
@@ -162,16 +165,33 @@ class SmartLink(ATLink):
         
         # We need to check if the self object has the reference_catalog attribute. It's an integration problem
         # with p4a that call this method when we don't have an internal link.
-        if hasattr(self, 'reference_catalog'): 
+        if hasattr(self, 'reference_catalog'):
             ilink = self.getInternalLink()
         else:
             ilink = None
         if ilink:
-            remote = ilink.absolute_url()
+            if queryUtility(ISmartlinkConfig,name="smartlink_config").relativelink:            
+                object = self.getField('internalLink').get(self)
+                remote = '/'.join(object.getPhysicalPath())
+                return quote(remote, safe='?$#@/:=+;$,&%')
+            else:
+                remote = ilink.absolute_url()
         else:
             remote = self.getExternalLink()
-        
+
         if not remote: remote = '' # ensure we have a string
+        backendlinks = queryUtility(ISmartlinkConfig,name="smartlink_config").backendlink
+        for backendlink in backendlinks:
+            if backendlink[backendlink.__len__()-1:]=='/':
+                blink = backendlink[:-1]
+            else:
+                blink = backendlink
+            if remote.startswith(blink):
+                frontendlinks = queryUtility(ISmartlinkConfig,name="smartlink_config").frontendlink
+                frontendlink = frontendlinks[backendlinks.index(backendlink)]
+                if frontendlink[frontendlink.__len__()-1:]=='/':
+                    frontendlink = frontendlink[:-1]
+                remote = remote.replace(blink,frontendlink)
         return quote(remote, safe='?$#@/:=+;$,&%')
 
     security.declarePrivate('cmf_edit')
@@ -203,4 +223,4 @@ class SmartLink(ATLink):
             interface.noLongerProvides(target, ISmartLinked)
         ATLink._processForm(self, data=data, metadata=metadata, REQUEST=REQUEST, values=values)
 
-atapi.registerType(SmartLink, PROJECTNAME)
+atapi.registerType(SmartLink, PROJECTNAME)
\ No newline at end of file
